@@ -576,13 +576,17 @@ class Job:
     return poll_fn
 
 
-def cleanup_expired_jobs(db=None, project=None, max_expire_days=30):
+def cleanup_expired_jobs(db=None,
+                         project=None,
+                         max_expire_days=30,
+                         max_timeout_days=1):
   """Clean up database entries for expired job statuses.
 
   Args:
     db: Firestore db instance.
     project: GCP project id.
     max_expire_days: Maximum expire days for a job.
+    max_timeout_days: Maximum running days for a job.
   """
   if not db:
     db = firestore.Client(project=project)
@@ -590,6 +594,7 @@ def cleanup_expired_jobs(db=None, project=None, max_expire_days=30):
   jobs_ref = db.collection(Job.JOB_STATUS_COLLECTION)
   now = datetime.datetime.now()
   expired_job_ids = []
+  timeout_job_ids = []
   # Find all expired jobs
   for job_ref in jobs_ref.stream():
     job = job_ref.to_dict()
@@ -597,6 +602,10 @@ def cleanup_expired_jobs(db=None, project=None, max_expire_days=30):
                                             '%Y-%m-%d-%H:%M:%S')
     if (now - start_time).days >= max_expire_days:
       expired_job_ids.append(job_ref.id)
+    elif job['status'] == JobStatus.RUNNING and (
+        now - start_time).days >= max_timeout_days:
+      logging.info('Job timeout: %s', job_ref.id)
+      timeout_job_ids.append(job_ref.id)
 
   logging.info('%d jobs to delete', len(expired_job_ids))
   # For each expired job, first delete all tasks inside the job,
@@ -609,3 +618,8 @@ def cleanup_expired_jobs(db=None, project=None, max_expire_days=30):
       task_ref.reference.delete()
     job_ref.delete()
     logging.info('Deleted expired job status: %s', job_id)
+
+  logging.info('%d jobs timeout', len(timeout_job_ids))
+  for job_id in timeout_job_ids:
+    job_ref = jobs_ref.document(job_id)
+    job_ref.update({'status': JobStatus.FAILED, 'error': 'timeout'})
